@@ -283,7 +283,7 @@ fn remove_comments<'a>(code: &'a str) -> String {
 ///
 fn get_incs<'a>(code: &'a str) -> Vec<Include> {
     let mut result = vec![];
-    let re = Regex::new(r#"[\n]{0,1}[\t\s]*(?P<captured>#include[\t\s]+["<].+[">])"#).unwrap();
+    let re = Regex::new(r#"(?P<captured>#include[\s]+["<].+[">])"#).unwrap();
     for cap in re.captures_iter(code) {
         result.push(Include {
             captured: cap.name("captured").unwrap().as_str().trim().to_string(),
@@ -296,8 +296,7 @@ fn get_incs<'a>(code: &'a str) -> Vec<Include> {
 ///
 fn get_local_vars<'a>(code: &'a str) -> Vec<LocalVariable> {
     let mut result = vec![];
-    let re =
-        Regex::new(r"[\n]{0,1}[\t\s]*(?i)(static)[\t\s\n]*(?P<captured>[^\(\{;=]+)[;=]").unwrap();
+    let re = Regex::new(r"(?i)(static)\s+(?P<captured>[^\(\{;=]+)[;=]").unwrap();
     for cap in re.captures_iter(code) {
         let name = cap
             .name("captured")
@@ -333,9 +332,9 @@ fn get_local_vars<'a>(code: &'a str) -> Vec<LocalVariable> {
 fn get_fncs<'a>(code: &'a str) -> Vec<Function> {
     let mut result = vec![];
     let re = Regex::new(
-        r"[\n]*[\s\t]*(?P<return_func>([a-zA-Z_][a-zA-Z_0-9]*\s+)+([a-zA-Z_][a-zA-Z_0-9]*\s*))\((?P<args>[^=!><>;\(\)-]*)\)[\s\t\r\n]*\{",
+        r"(?P<return>[[:alpha:]_][\w]*\s+)+(?P<name>[[:alpha:]_][\w]*)\s*\((?P<args>[^=!><>;\(\)-]*)\)\s*\{",
     ).unwrap();
-    let get_types = |args: &'a str| -> String {
+    let get_atypes = |args: String| -> String {
         let mut type_list = String::new();
         let mut first_pos = true;
         let arg_list = args.split(',').collect::<Vec<&str>>();
@@ -350,9 +349,11 @@ fn get_fncs<'a>(code: &'a str) -> Vec<Function> {
                 let mut tmpstr = arg.get(..(pos + 1)).unwrap().trim().to_string();
                 let re4const = Regex::new(r"\w[\s\r\n]+const[\s\r\n]*\*").unwrap();
                 if let Some(_) = re4const.captures(&tmpstr) {
+                    let re4space = Regex::new(r"\s+").unwrap();
                     let pos = tmpstr.find("const").unwrap();
                     tmpstr.replace_range(pos..(pos + 5), "");
                     tmpstr = format!("const {}", tmpstr);
+                    tmpstr = re4space.replace_all(&tmpstr, " ").to_string();
                 }
                 type_list.push_str(&tmpstr);
                 let array_dimension = arg.get(pos..).unwrap().matches("[").count();
@@ -365,59 +366,31 @@ fn get_fncs<'a>(code: &'a str) -> Vec<Function> {
         type_list
     };
     for cap in re.captures_iter(code) {
-        let name = cap
-            .name("return_func")
-            .unwrap()
-            .as_str()
-            .trim()
-            .split(' ')
-            .collect::<Vec<&str>>()
-            .last()
-            .unwrap()
-            .trim_matches('*')
-            .to_string();
-        let return_type = cap
-            .name("return_func")
-            .unwrap()
-            .as_str()
-            .trim()
-            .strip_suffix(&name)
-            .unwrap()
-            .replace("static", "")
-            .replace("STATIC", "")
-            .replace("inline", "")
-            .replace("INLINE", "")
-            .trim()
-            .to_string();
-        let is_local = cap
-            .name("return_func")
-            .unwrap()
-            .as_str()
-            .contains("static ")
-            || cap
-                .name("return_func")
-                .unwrap()
-                .as_str()
-                .contains("STATIC ");
         let re4space = Regex::new(r"\s+").unwrap();
-        let mut args = cap
-            .name("args")
-            .unwrap()
-            .as_str()
-            .trim()
-            .to_string()
-            .replace("\n", "")
-            .replace("\r", "");
-        args = re4space.replace_all(&args, " ").to_string();
-        let mut args_type = get_types(cap.name("args").unwrap().as_str());
-        args_type = re4space.replace_all(&args_type, " ").to_string();
+        let raw_args = re4space
+            .replace_all(cap.name("args").unwrap().as_str().trim(), " ")
+            .to_string();
         result.push(Function {
             captured: cap.get(0).unwrap().as_str().trim().to_string(),
-            name: name,
-            is_local: is_local,
-            rtype: return_type,
-            args: args,
-            atypes: args_type,
+            name: cap.name("name").unwrap().as_str().trim().to_string(),
+            is_local: cap
+                .get(0)
+                .unwrap()
+                .as_str()
+                .to_ascii_lowercase()
+                .contains("static"),
+            rtype: cap
+                .name("return")
+                .unwrap()
+                .as_str()
+                .replace("static", "")
+                .replace("STATIC", "")
+                .replace("inline", "")
+                .replace("INLINE", "")
+                .trim()
+                .to_string(),
+            args: raw_args.clone(),
+            atypes: get_atypes(raw_args.clone()),
         });
     }
     result
@@ -601,19 +574,22 @@ int func1()
 {
     return 0;
 }
-void func2(int a)
+void func2(int const * a)
 {
 }
 ";
         let temp = "\
 // functions
 @fncs@@rtype@ @name@(@args@);
+@atypes@
 @end-fncs@
 ";
         let expected = "\
 // functions
 int func1();
-void func2(int a);
+
+void func2(int const * a);
+const int *
 
 ";
         let parser = Parser::parse(code);
@@ -626,21 +602,23 @@ void func2(int a);
         let sourcename = "test";
         let code = "\
 // functions
-int func1()
+void func1()
 {
-    return 0;
+    return;
 }
-void func2(int a)
+int func2(int a)
 {
-    func1();
+    return func1();
 }
 ";
         let temp = "\
 @ncls@- @caller.name@ -> @callee.name@
+    - return @callee.rtype.value(0)@;
 @end-ncls@
 ";
         let expected = "\
 - func2 -> func1
+    - return ;
 
 ";
         let parser = Parser::parse(code);
